#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/timeb.h>
#include <signal.h>
#include <time.h>
#include <pcap.h>
#include "iphash.h"
#include "lib.h"

static struct timeb starttime;
static struct timeb lasttime;
unsigned int delay = 1;
unsigned int prefix = 14;
int counter = -1;
int human = 0;
int bits = 0;
int zero = 0;
struct ipclassinfo * ipci = NULL;
int printtop = 10;


void host_inetntoa(unsigned char * str, int max, u_int32_t ip)
{
	snprintf(str, max, "%d.%d.%d.%d", (ip >> 24) & 0xff, (ip >> 16) & 0xff, (ip >> 8) & 0xff, ip & 0xff);
}

void show(struct top * top, unsigned char * header, unsigned long millitm)
{
	int n, i, left = 0, right;
	unsigned char unit[5];
	
	printf("%s:\n", header);
	n = top->n / 2;
	right = n;
	
	if(bits)
		snprintf(unit, 5, "bps");
	else
		snprintf(unit, 5, "Bps");
	
	for(i = 0; i<n; i++)
	{
		unsigned char leftip[16];
		unsigned char rightip[16];
		unsigned char leftstatus[18];
		unsigned char rightstatus[18];
		double lv;
		double rv;
		
		lv = top->top[left].data;
		rv = top->top[right].data;
		lv /= millitm; rv /= millitm;
		lv *= 1000; rv *= 1000;
		if(bits)
		{
			lv *= 8;
			rv *= 8;
		}
		
		host_inetntoa(leftip, 16, top->top[left].ip);
		host_inetntoa(rightip, 16, top->top[right].ip);
		mkhuman(lv, 18, leftstatus, unit, 1024, human);
		mkhuman(rv, 18, rightstatus, unit, 1024, human);
		
		if(!top->top[left].ip) continue;
		if(!top->top[right].ip)
			printf("%3d.%15s %18s\n", left + 1, leftip, leftstatus);
		else 
			printf("%3d.%15s %18s %3d.%15s %18s\n", left + 1, leftip, leftstatus, right + 1, rightip, rightstatus);
		
		left++; right++;
		
	}
}


void output(int signal)
{
	time_t time;
	unsigned short millitm;
	unsigned long miliFromLast, miliFromStart;
	struct top * intop;
	struct top * outtop;

	alarm(delay);
	
	time = lasttime.time;
	millitm = lasttime.millitm;
	
	ftime(&lasttime);
	
	miliFromStart = (lasttime.time - starttime.time) * 1000 + lasttime.millitm - starttime.millitm;
	miliFromLast = (lasttime.time - time) * 1000 + lasttime.millitm - millitm;

	intop = findtop(printtop, 1);
	outtop = findtop(printtop, 0);

	if(zero)
	{
		show(intop, "Input bandwidth abusers:", miliFromLast);
		show(outtop, "Output bandwidth abusers:", miliFromLast);
	}
	else
	{
		show(intop, "Input bandwidth abusers:", miliFromStart);
		show(outtop, "Output bandwidth abusers:", miliFromStart);
	}
	
	free(intop->top); free(intop);
	free(outtop->top); free(outtop);

	counter--;
	if(counter == 0) exit(0);
	if(zero) zero_entries();
}


void help(unsigned char * name)
{
	printf("bmtools 0.5 - Mateusz 'mteg' Golicz <mtg@elsat.net.pl>, 2003\n", name);
	printf("usage: %s [options]\n", name);
	printf("  -h        Show this help\n");
	printf("  -t <t>    Print <t> top (even!) abusers - default 10\n");
	printf("  -u        More human-readable output\n");
	printf("  -z        Zero counters after each output (calculate current, not average)\n");
	printf("  -b        Show output in bit/s\n");
	printf("  -c <class>Consider nodes from this class (REQUIRED)\n");
	printf("  -p <pref> Datalink prefix size (gets substracted from each packet size, default: 14 (ethernet))\n");
	printf("            This is REQUIRED if the interface is not ethernet\n");
	printf("  -i <int>  Use interface <int> - default eth0\n");
	printf("  -f <bpf>  BPF filter expression to use\n");
	printf("  -r <t>    Print results every <t> seconds (default: 1)\n");
	printf("  -q <n>    Quit after printing n results\n");
}

void counter_callback(unsigned char *userdata,const struct pcap_pkthdr *hd,const unsigned char *data)
{
	struct iphdr *iph;
	u_int32_t sip;
	u_int32_t dip;
	u_int32_t host;
	long long in = 0, out = 0;

	iph=(struct iphdr*) (unsigned char*) (((unsigned char*) data)+prefix);
			
	if(iph->version != 4) return;

	sip = iph->saddr;
	dip = iph->daddr;

 	if(is_ours(ipci, sip))
        	{ host = sip; out = hd->caplen - prefix;}
	    
	if(is_ours(ipci, dip))
		{ host = dip; in = hd->caplen - prefix;	}
	
	if(!(in || out)) return;
	if(in && out) return;
	add_entry(ntohl(host), in, out);
		
}


int main(int argc, char ** argv)
{
	unsigned char * bpfexpression = NULL;
	unsigned char * interface = "eth0";
	char ebuff[PCAP_ERRBUF_SIZE];
	pcap_t *handle;
	struct bpf_program bpf;
	int c;
	
	while((c = getopt(argc, argv, "t:hi:f:r:up:c:bq:z")) != EOF)
	{
		switch(c)
		{
			case 'u': human = 1; break;
			case 'z': zero = 1; break;
			case 'b': bits = 1; break;
			case 'q': counter = atoi(optarg); break;
			case 't':
				printtop = atoi(optarg);
				if(printtop < 1) printtop = 1;
				break;
			case 'c':
				ipci = add_class(optarg, ipci);
				if(!ipci) return(1);
				break;
			case 'p': prefix = atoi(optarg); break;
			case 'h':
				help(argv[0]);
				exit(0);
			case 'i':
				interface = strdup(optarg);
				break;
			case 'f':
				bpfexpression = strdup(optarg);
				break;
			case 'r':
				delay = atoi(optarg);
				if(delay < 1) delay = 1;
				break;
		}
	}
	
	if(!ipci)
	{
		fprintf(stderr, "No classes specified. Type %s -h\n", argv[0]);
		return(1);
	}
	
	signal(SIGPIPE, SIG_IGN);
	signal(SIGALRM, output);
	if((handle = pcap_open_live(interface, 2048, -1, 50, ebuff))==NULL)
	{
		fprintf(stderr, "pcap_open_live(): %s\n", ebuff);
		return(0);
 	}

	if(bpfexpression)
	{
		if(pcap_compile(handle, &bpf, bpfexpression, 1, 0xffffffff) == -1)
		{
			fprintf(stderr, "pcap_compile(): %s\n", pcap_geterr(handle));
			return(1);
		}
		if(pcap_setfilter(handle, &bpf) == -1)
		{
			fprintf(stderr, "pcap_compile(): %s\n", pcap_geterr(handle));
			return(1);
		}
	}

	
	alarm(delay);
	ftime(&starttime);
	ftime(&lasttime);
	pcap_loop(handle, -1, counter_callback, NULL);
	pcap_close(handle);
}
