#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include "iphash.h"
#define CLASS_SIZE (256 * sizeof(struct entry*))

static struct entry ***** root_hash = NULL;
static unsigned int used_memory = 0;

extern int verbose_dump;

u_int32_t inet_addr(unsigned char*);
unsigned char * inet_ntoa(u_int32_t);

struct ipclassinfo * add_class(char * name, struct ipclassinfo * ci)
{
    char * tmpname = strdup(name);
    char * mask;
    int nmask;
    int i;
    u_int32_t ip;
    u_int32_t netmask;
    struct ipclassinfo *ret;
    
    if(mask = index(tmpname, '/')) 
	nmask = atoi(mask + 1);
    else
    {
	fprintf(stderr, "Class notation: a.b.c.d/mask\n");
	free(tmpname); return(NULL);
    }
    *mask = 0;
    ip = inet_addr(tmpname);
    free(tmpname);
    netmask = 0;
    for(i = 0; i<nmask; i++) netmask = (netmask >> 1) | 0x80000000;
    netmask = htonl(netmask);
#ifdef DEBUG
    fprintf(stderr, "tracing class: %s (%x) ", inet_ntoa(ip), ip);
    fprintf(stderr, "netmask %s (%x)\n", inet_ntoa(netmask), netmask);
#endif
    ret = (struct ipclassinfo*) malloc(sizeof(struct ipclassinfo));
    ret->next = ci;
    ret->net = ip;
    ret->mask = netmask;
    return(ret);
}

int is_ours(struct ipclassinfo * ci, u_int32_t host)
{
    for(; ci; ci = ci->next)
	if((host & ci->mask) == ci->net) return(1);

    return(0);
}

static inline void * bzalloc(int len)
{
    void * ret;
    ret = (void*) malloc(len);
    memset(ret, 0, len);
    used_memory += len;
#ifdef DEBUG
    printf("alloc(%d)\n", len);
#endif
    return(ret);
}

struct entry * get_entry(u_int32_t ip)
{
    struct entry ***** ahash;
    struct entry **** bhash;
    struct entry *** chash;
    struct entry ** dhash;
    struct entry * host;
    int a = ip >> 24;
    int b = (ip >> 16) & 0xff;
    int c = (ip >> 8) & 0xff;
    int d = ip & 0xff;
    if(!(ahash = root_hash)) ahash = root_hash = (struct entry *****) bzalloc(CLASS_SIZE);
    if(!(bhash = ahash[a])) bhash = ahash[a] = (struct entry ****) bzalloc(CLASS_SIZE);
    if(!(chash = bhash[b])) chash = bhash[b] = (struct entry ***) bzalloc(CLASS_SIZE);
    if(!(dhash = chash[c])) dhash = chash[c] = (struct entry **) bzalloc(CLASS_SIZE);
    if(!(host = dhash[d])) host = dhash[d] = (struct entry*) bzalloc(sizeof(struct entry));
    return(host);
}

void add_entry(u_int32_t ip, long long in, long long out)
{
    struct entry * entry = get_entry(ip);
    entry->input += in;
    entry->output += out;
#ifdef DEBUG
    fprintf(stderr, "Added %Ld, %Ld bytes to entry %s Now counters are %Ld, %Ld\n",  in, out, inet_ntoa(htonl(ip)),entry->input, entry->output);
#endif
}


void zero_entries(void)
{
    unsigned int i, j, k, l;
    struct entry ***** hasha = root_hash;
    if(!hasha) return;
    
    for(i = 0; i<256; i++)
    {
	struct entry **** hashb = hasha[i];
	if(!hashb) continue;
	for(j = 0; j<256; j++)
	{
	    struct entry *** hashc = hashb[j];
	    if(!hashc) continue;
	    for(k = 0; k<256; k++)
	    {
		struct entry ** hashd = hashc[k];
		if(!hashd) continue;
		for(l = 0; l<256; l++)
		{
		    struct entry * host = hashd[l];
		    if(host)
		    {
			host->input = 0;
			host->output = 0;
		    }
		}
	    }
	}
    }
}


				

struct top * findtop(int n, int crit)
{
    unsigned int i, j, k, l, m;
    struct top * top = malloc(sizeof(struct top));
    struct entry ***** hasha = root_hash;
    
    top -> n = n;
    top -> top = (struct singletop*) malloc(sizeof(struct singletop) * n);
    for(m = 0; m < n; m++) 
    {
    	top->top[m].data = 0;
	top->top[m].ip = 0;
    }

    if(!hasha) return(top);
    
    for(i = 0; i<256; i++)
    {
	struct entry **** hashb = hasha[i];
	if(!hashb) continue;
	for(j = 0; j<256; j++)
	{
	    struct entry *** hashc = hashb[j];
	    if(!hashc) continue;
	    for(k = 0; k<256; k++)
	    {
		struct entry ** hashd = hashc[k];
		if(!hashd) continue;
		for(l = 0; l<256; l++)
		{
		    struct entry * host = hashd[l];
		    u_int32_t ip;
		    if(host)
		    {
		    	long long value;
			if(crit)
				value = host->input;
			else
				value = host->output;
			
			for(m = 0; m<n; m++)
			{
				if(value > top->top[m].data)
				{
					if(m < (n - 1))
						memmove(&top->top[m + 1], &top->top[m], sizeof(struct singletop) * (n - m - 1));
	
					top->top[m].data = value;
					top->top[m].ip = (i << 24) | (j << 16) | (k << 8) | l;
					break;
				}
			}
		    }
		}
	    }
	}
    }
    return(top);
}
