#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/timeb.h>
#include <signal.h>
#include <time.h>
#include <pcap.h>
#include "lib.h"

static struct timeb starttime;
static struct timeb lasttime;
unsigned long bytes;
unsigned long packets;
unsigned long long totalbytes;
unsigned long long totalpackets;
unsigned int delay = 1;
unsigned int prefix = 0;
unsigned int bits = 0;
int counter = -1;
int human = 0;

void output(int signal)
{
	time_t time;
	unsigned short millitm;
	unsigned long nowbytes, nowpackets;
	unsigned long miliFromLast, miliFromStart;
	double bpsFromStart, bpsFromLast, ppsFromStart, ppsFromLast;
	char bpsFromLastStr[100];
	char bpsFromStartStr[100];
	char ppsFromLastStr[100];
	char ppsFromStartStr[100];

	alarm(delay);
	
	time = lasttime.time;
	millitm = lasttime.millitm;
	
	nowbytes = bytes; bytes = 0;
	nowpackets = packets; packets = 0;
	ftime(&lasttime);
	
	totalbytes += nowbytes;
	totalpackets += nowpackets;
	
	miliFromStart = (lasttime.time - starttime.time) * 1000 + lasttime.millitm - starttime.millitm;
	miliFromLast = (lasttime.time - time) * 1000 + lasttime.millitm - millitm;
	
	ppsFromStart = totalpackets; ppsFromStart /= miliFromStart;
	ppsFromLast = nowpackets; ppsFromLast /= miliFromLast;
	bpsFromStart = totalbytes; bpsFromStart /= miliFromStart;
	bpsFromLast = nowbytes; bpsFromLast /= miliFromLast;
	ppsFromStart *= 1000; ppsFromLast *= 1000;
	bpsFromStart *= 1000; bpsFromLast *= 1000;
	
	if(bits)
		mkhuman(bpsFromLast * 8, 100, bpsFromLastStr, "bps", 1024, human);
	else
		mkhuman(bpsFromLast, 100, bpsFromLastStr, "Bps", 1024, human);
		
	mkhuman(ppsFromLast, 100, ppsFromLastStr, "pps", 1000, human);
	
	if(bits)
		mkhuman(bpsFromStart * 8, 100, bpsFromStartStr, "bps", 1024, human);
	else
		mkhuman(bpsFromStart, 100, bpsFromStartStr, "Bps", 1024, human);

	mkhuman(ppsFromStart, 100, ppsFromStartStr, "pps", 1000, human);
	printf("-> Currently %s/%s, Average: %s/%s\n", bpsFromLastStr, ppsFromLastStr, bpsFromStartStr, ppsFromStartStr);
	counter--;
	if(counter == 0) exit(0);
}


void help(unsigned char * name)
{
        printf("bmtools 0.5 - Mateusz 'mteg' Golicz <mtg@elsat.net.pl>, 2003\n", name);
	printf("usage: %s [options]\n", name);
	printf("  -h        Show this help\n");
	printf("  -u        More human-readable output\n");
	printf("  -b        Show output in bps (not Bps)\n");
	printf("  -p <pref> Datalink prefix (gets substracted from each packet size, default: 0)\n");
	printf("  -i <int>  Use interface <int> - default eth0\n");
	printf("  -f <bpf>  BPF filter expression to use\n");
	printf("  -r <t>    Print results every <t> seconds (default: 1)\n");
	printf("  -q <r>    Quit after printing r results\n");
}

void counter_callback(unsigned char *userdata,const struct pcap_pkthdr *hd,const unsigned char *data)
{
	bytes += hd->caplen - prefix;
	packets += 1;
}


int main(int argc, char ** argv)
{
	unsigned char * bpfexpression = NULL;
	unsigned char * interface = "eth0";
	char ebuff[PCAP_ERRBUF_SIZE];
	pcap_t *handle;
	struct bpf_program bpf;
	int c;
	
	
	while((c = getopt(argc, argv, "hi:f:r:up:bq:")) != EOF)
	{
		switch(c)
		{
			case 'u': human = 1; break;
			case 'b': bits = 1; break;
			case 'q': counter = atoi(optarg) ; break;
			case 'p': prefix = atoi(optarg); break;
			case 'h':
				help(argv[0]);
				exit(0);
			case 'i':
				interface = strdup(optarg);
				break;
			case 'f':
				bpfexpression = strdup(optarg);
				break;
			case 'r':
				delay = atoi(optarg);
				if(delay < 1) delay = 1;
				break;
		}
	}
	signal(SIGPIPE, SIG_IGN);
	signal(SIGALRM, output);
	if((handle = pcap_open_live(interface, 2048, -1, 50, ebuff))==NULL)
	{
		fprintf(stderr, "pcap_open_live(): %s\n", ebuff);
		return(0);
 	}

	if(bpfexpression)
	{
		if(pcap_compile(handle, &bpf, bpfexpression, 1, 0xffffffff) == -1)
		{
			fprintf(stderr, "pcap_compile(): %s\n", pcap_geterr(handle));
			return(1);
		}
		if(pcap_setfilter(handle, &bpf) == -1)
		{
			fprintf(stderr, "pcap_compile(): %s\n", pcap_geterr(handle));
			return(1);
		}
	}

	
	alarm(delay);
	ftime(&starttime);
	ftime(&lasttime);
	pcap_loop(handle, -1, counter_callback, NULL);
	pcap_close(handle);
}
