#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/timeb.h>
#include <signal.h>
#include <time.h>
#include <pcap.h>
#include "rated.h"
#include "lib.h"

struct rated * r;
int verbose = 0;

struct rated * readconfig(unsigned char*);
int ratedServer(struct rated * r);

void calculate(int signal)
{
	int i;
	time_t lastt;
	unsigned short lastm;
	
	alarm(r->interval);
	
	lastt = r->lastTime.time;
	lastm = r->lastTime.millitm;
	
	ftime(&r->lastTime);
	
	for(i = 0; i < r->sp_count; i++)
	{
		unsigned long miliFromLast;
		unsigned long bytesFromLast, packetsFromLast;
		
		miliFromLast = (r->lastTime.time - lastt) * 1000 + r->lastTime.millitm - lastm;
		
		bytesFromLast = r->entries[i]->overallBytes - r->entries[i]->lastBytes;
		r->entries[i]->lastBytes = r->entries[i]->overallBytes;

		packetsFromLast = r->entries[i]->overallPackets - r->entries[i]->lastPackets;
		r->entries[i]->lastPackets = r->entries[i]->overallPackets;

		r->entries[i]->currentBytes = bytesFromLast;
		r->entries[i]->currentBytes *= 1000;
		r->entries[i]->currentBytes /= miliFromLast;

		r->entries[i]->currentPackets = packetsFromLast;
		r->entries[i]->currentPackets *= 1000;
		r->entries[i]->currentPackets /= miliFromLast;
		
		if(verbose)
			printf("After %d ms: Channel %s: %.2f Bps %.2f pps\n", miliFromLast, r->entries[i]->shortname, r->entries[i]->currentBytes, r->entries[i]->currentPackets);
	}
}


void help(unsigned char * name)
{
        printf("bmtools 0.7 - Mateusz 'mteg' Golicz <mtg@elsat.net.pl>, 2003\n", name);
	printf("usage: %s [options]\n", name);
	printf("  -h        Show this help\n");
	printf("  -v        Verbose: show configured values after parsing config file\n");
	printf("  -c <file> Config file path - required\n");
}

void counter_callback(unsigned char *userdata, const struct pcap_pkthdr *hd, const unsigned char *data)
{
	int bytes, i, n;
	bytes = hd->caplen - r->prefix;
	for(i = 0, n = r->sp_count; i < n; i++)
	{
		if(bpf_filter(r->entries[i]->filter.bf_insns, data, hd->len, hd->caplen))
		{
			r->entries[i]->overallBytes += bytes;
			r->entries[i]->overallPackets += 1;
		}
	}
}


int main(int argc, char ** argv)
{
	char ebuff[PCAP_ERRBUF_SIZE];
	pcap_t *handle;
	int c;
	unsigned char * confname = NULL;
	struct bpf_program mainbpf;
	int i;
	
	
	while((c = getopt(argc, argv, "hc:v")) != EOF)
	{
		switch(c)
		{
			case 'h': help(argv[0]); exit(0);
			case 'c': confname = strdup(optarg); break;
			case 'v': verbose = 1; break;
		}
	}
	
	if(!confname) { help(argv[0]); exit(0); }
	
	r = readconfig(confname);
	if(!r) exit(0);

	if(!r->sp_count)
	{
		fprintf(stderr, "No data sources (rules). Nothing to do.");
		return(0);
	}

	signal(SIGPIPE, SIG_IGN);
	signal(SIGALRM, calculate);
	if((handle = pcap_open_live(r->interface, r->mtu, -1, 50, ebuff))==NULL)
	{
		fprintf(stderr, "pcap_open_live(): %s\n", ebuff);
		return(0);
 	}

	if(!r->prefix) r->prefix = llheader(pcap_datalink(handle));

	if(r->general)
	{
		if(pcap_compile(handle, &mainbpf, r->general, 1, 0xffffffff) == -1)
		{
			fprintf(stderr, "pcap_compile('%s'): %s\n", r->general, pcap_geterr(handle));
			return(1);
		}
		if(pcap_setfilter(handle, &mainbpf) == -1)
		{
			fprintf(stderr, "pcap_compile('%s'): %s\n", r->general, pcap_geterr(handle));
			return(1);
		}
	}
	
	for(i = 0; i<r->sp_count; i++)
	{
		if(pcap_compile(handle, &r->entries[i]->filter, r->entries[i]->bpf_rule, 1, 0xffffffff) == -1) 
		{
			fprintf(stderr, "pcap_compile('%s'): %s\n", r->entries[i]->bpf_rule, pcap_geterr(handle));
			return(1);
		}					
	}

	if(ratedServer(r))
	{
		fprintf(stderr, "Unable to create server thread");
		return(1);
	}
	

	alarm(r->interval);
	ftime(&r->startTime);
	ftime(&r->lastTime);
	pcap_loop(handle, -1, counter_callback, NULL);
	pcap_close(handle);
}
