
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lib.h"
#include "rated.h"
#define MAX_LINESIZE 2048
#define MAX_ERRORSIZE 512
static char default_interface[] = "eth0";

struct cdir
{
	unsigned char * name;
	int ac;
	int (*set)(struct rated *, unsigned char **, unsigned char *);
};


static int setlisten(struct rated * conf, unsigned char ** args, unsigned char  *error)
{
	int z;
	z = atoi(args[1]);
	if(z <= 0 || z >= 65536)
	{
		snprintf(error, MAX_ERRORSIZE, "Invalid port number");
		return(1);
	}
	else
	{
		conf->udpport = z;
		return(0);
	}
}

static int grant(struct rated * conf, unsigned char ** args, unsigned char  *error)
{
	u_int32_t host, netmask;
	struct grant * g;
	
	if(parseaddr(args[1], &host, &netmask))
	{
		snprintf(error, MAX_ERRORSIZE, "Wrong IP prefix notation");
		return(1);
	}
	
	if((host & netmask) != host)
	{
		snprintf(error, MAX_ERRORSIZE, "Wrong prefix (non-network bits set in address)");
		return(1);
	}
	g = (struct grant*) malloc(sizeof(struct grant));

	g->next = conf->acl;
	g->ip = host;
	g->mask = netmask;
	conf->acl = g;
	return(0);
}

static int overall(struct rated * conf, unsigned char ** args, unsigned char  *error)
{
	conf->general = strdup(args[1]);
	return(0);
}

static int interface(struct rated * conf, unsigned char ** args, unsigned char  *error)
{
	conf->interface = strdup(args[1]);
	return(0);
}

static int prefix(struct rated * conf, unsigned char ** args, unsigned char  *error)
{
	conf->prefix = atoi(args[1]);
	return(1);
}

static int rule(struct rated * conf, unsigned char ** args, unsigned char  *error)
{
	int c;
	struct statpoint * entry;
	c = wc(args);
	if(c < 4)
	{
		snprintf(error, MAX_ERRORSIZE, "Too few arguments");
		return(1);
	}
	
	if(c > 4)
	{
		if(c > 5 || (strcmp("max", args[2]) && strcmp("yellow", args[2]) && strcmp("red", args[2])))
		{
			snprintf(error, MAX_ERRORSIZE, "Too many arguments");
			return(1);
		}
	}
	
	if(strcmp("max", args[2]) && strcmp("yellow", args[2]) && strcmp("red", args[2]))
	{
		entry = (struct statpoint*) malloc(sizeof(struct statpoint));
		entry->shortname = strdup(args[1]);
		entry->description = strdup(args[3]);
		entry->bpf_rule = strdup(args[2]);
		entry->lastBytes = 0; entry->lastPackets = 0;
		entry->overallBytes = 0; entry->overallPackets = 0;
		entry->currentBytes = entry->currentPackets = entry->maxBytes = entry->maxPackets = 0;
		entry->redBytes = entry->redPackets = entry->yellowBytes = entry->yellowPackets = 0;
		conf->entries = (struct statpoint**) realloc(conf->entries, sizeof(struct statpoint*) * (conf->sp_count + 1));
		conf->entries[conf->sp_count++] = entry;
		return(0);
	}
	else
	{
		double bps, pps;
		int i;
		entry = NULL;
		for(i = 0; i<conf->sp_count; i++)
			if(strcmp(args[1], conf->entries[i]->shortname) == 0) break;
		
		if(i == conf->sp_count)
		{
			snprintf(error, MAX_ERRORSIZE, "No such rule");
			return(1);
		}
		entry = conf->entries[i];
		
		if(human2machine(args[3], &bps) || human2machine(args[4], &pps))
		{
			snprintf(error, MAX_ERRORSIZE, "Invalid rate specification");
			return(1);
		}
		
		if(strcmp("red", args[2]) == 0) { entry->redPackets = pps; entry->redBytes = bps; }
		if(strcmp("yellow", args[2]) == 0) { entry->yellowPackets = pps; entry->yellowBytes = bps; }
		if(strcmp("max", args[2]) == 0) { entry->maxPackets = pps; entry->maxBytes = bps; }
		return(0);
	}
}



static struct cdir directives[] = {{"listen", 2, setlisten},
			    	   {"grant", 2, grant},
			    	   {"interface", 2, interface},
				   {"overall", 2, overall},
				   {"prefix", 2, prefix},
				   {"rule", 0, rule},
				   {NULL, 0, NULL}};

struct rated * readconfig(unsigned char * name)
{
	struct rated * conf;
	FILE * fh;
	int line;
	
	if((fh = fopen(name, "r")))
	{
		unsigned char linebuffer[MAX_LINESIZE];
		
		conf = (struct rated*) malloc(sizeof(struct rated));

		conf->sp_count = 0;
		conf->entries = NULL;
		conf->prefix = 0;
		conf->interval = 1;
		conf->udpport = 6000;
		conf->interface = default_interface;
		conf->general = NULL;
		conf->acl = NULL;
		conf->mtu = 2048;
		line = 0;
		while(fgets(linebuffer, MAX_LINESIZE, fh))
		{
			unsigned char ** words;
			unsigned char * eol;
			
			line ++;
			if(linebuffer[0] == '#') continue;
			if((eol = index(linebuffer, '\n'))) * eol = 0;
			if(!strlen(linebuffer)) continue;
			
			words = explode(linebuffer);
			if(words)
			{
				int i, ac = wc(words);
				unsigned char errbuff[MAX_ERRORSIZE];
//				for(i = 0; words[i]; i++) printf("words[%d] = '%s' ", i, words[i]);
//				printf("\n");
				for(i = 0; directives[i].name; i++)
				{
					if(strcmp(directives[i].name, words[0]) == 0)
					{
						if(directives[i].ac != ac && directives[i].ac)
						{
							fprintf(stderr, "%s: line %d: invalid argument count.\n", name, line);
							return(NULL); /* config is not discarded */
						}
						if(directives[i].set(conf, words, errbuff))
						{
							fprintf(stderr, "%s: line %d: %s\n", name, line, errbuff);
							return(NULL); /* config is not discarded */
						}
						break;
					}
				}
				if(!directives[i].name)
				{
					fprintf(stderr, "%s: line %d: syntax error.\n", name, line);
					return(NULL); /* config is not discarded */
				}
			}
			freewords(words);
		}
		fclose(fh);
		return(conf);
	}
	else
		return(NULL);
}
