
#include <pthread.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include "rated.h"
#include "ratednet.h"
#include "lib.h"

#define MAXPACKET 1200

#define RETRIES 3
#define RETRYTIMEOUT 750000

#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#endif

int rtn_count(u_int32_t ip, int port, int verbose)
{
	int s, r;
	struct sockaddr_in sin;
	unsigned char pktbuff[MAXPACKET];
	unsigned char rcvbuff[MAXPACKET];
	
	s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if(s == -1)
	{
		perror("socket");
		exit(1);
	}
	
	sin.sin_family = PF_INET;
	sin.sin_port = htons(port);
	sin.sin_addr.s_addr = ip;

	pktbuff[0] = REQUEST_DSCOUNT;
	
	r = RETRIES;
	while(r--)
	{
		fd_set fds;
		struct timeval tv;
		
		if(verbose) printf("Sending request...\n");
		if(sendto(s, pktbuff, 1, 0, &sin, sizeof(struct sockaddr_in)) == -1)
		{
			perror("sendto");
			close(s);
			return 0;
		}
		FD_ZERO(&fds);
		FD_SET(s, &fds);
		tv.tv_sec = 0;
		tv.tv_usec = RETRYTIMEOUT;
		if(select(s + 1, &fds, NULL, NULL, &tv))
		{ 
			int n;
			n = recv(s, rcvbuff, MAXPACKET, 0); /* TODO: check whether packet comes from the right host */
			if(n == -1)
			{
				perror("recv");
				close(s);
				return(0);
			}
			if(rcvbuff[0] == RESPONSE_DSCOUNT)
			{
				if(rcvbuff[1] != PROTOCOL_VERSION)
				{
					if(verbose) printf("Protocol mismatch.\n");
					close(s);
					return(0);
				}
				close(s);
				return(rcvbuff[2]);
			}
			else
				if(verbose) printf("Invalid reply.\n");
		}
		if(verbose) printf("Timeout.\n");
	}

	close(s);
	return(0);
}

int rtn_dsinfo(u_int32_t ip, int port, int verbose, int dsn, struct dseinfo * dse)
{
	int s, r;
	struct sockaddr_in sin;
	unsigned char pktbuff[MAXPACKET];
	unsigned char rcvbuff[MAXPACKET];
	
	s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if(s == -1)
	{
		perror("socket");
		exit(1);
	}
	
	sin.sin_family = PF_INET;
	sin.sin_port = htons(port);
	sin.sin_addr.s_addr = ip;

	pktbuff[0] = REQUEST_DSINFO;
	pktbuff[1] = dsn;
	
	r = RETRIES;
	while(r--)
	{
		fd_set fds;
		struct timeval tv;
		
		if(verbose) printf("Sending request...\n");
		if(sendto(s, pktbuff, 2, 0, &sin, sizeof(struct sockaddr_in)) == -1)
		{
			perror("sendto");
			close(s);
			return 0;
		}
		FD_ZERO(&fds);
		FD_SET(s, &fds);
		tv.tv_sec = 0;
		tv.tv_usec = RETRYTIMEOUT;
		if(select(s + 1, &fds, NULL, NULL, &tv))
		{ 
			int n;
			n = recv(s, rcvbuff, MAXPACKET, 0); /* TODO: check whether packet comes from the right host */
			if(n == -1)
			{
				perror("recv");
				close(s);
				return(0);
			}
			if(rcvbuff[0] == RESPONSE_DSINFO)
			{
				if(rcvbuff[1] == dsn)
				{
					memcpy((unsigned char *) dse, rcvbuff + 2, sizeof(struct dsinfo));
					dse->shortname = strdup(rcvbuff + 2 + sizeof(struct dsinfo)); /* TODO: more safety */
					dse->description = strdup(rcvbuff + 2 + sizeof(struct dsinfo) + strlen(dse->shortname) + 1);
					close(s);
					return(1);		
				}
			}
			else
				printf("Invalid reply.\n");
		}
		printf("Timeout.\n");
	}

	close(s);
	return(0);
}

int rtn_poll(u_int32_t ip, int port, int verbose, int dsn, struct dsdata * dsd, time_t * stamp)
{
	int s, r;
	struct sockaddr_in sin;
	unsigned char pktbuff[MAXPACKET];
	unsigned char rcvbuff[MAXPACKET];
	
	s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if(s == -1)
	{
		perror("socket");
		exit(1);
	}
	
	sin.sin_family = PF_INET;
	sin.sin_port = htons(port);
	sin.sin_addr.s_addr = ip;

	pktbuff[0] = REQUEST_DSDATA;
	pktbuff[1] = 1;
	pktbuff[2] = dsn;
	
	r = RETRIES;
	while(r--)
	{
		fd_set fds;
		struct timeval tv;
		
		if(verbose) printf("Sending request...\n");
		if(sendto(s, pktbuff, 3, 0, &sin, sizeof(struct sockaddr_in)) == -1)
		{
			perror("sendto");
			close(s);
			return 0;
		}
		FD_ZERO(&fds);
		FD_SET(s, &fds);
		tv.tv_sec = 0;
		tv.tv_usec = RETRYTIMEOUT;
		if(select(s + 1, &fds, NULL, NULL, &tv))
		{ 
			int n;
			n = recv(s, rcvbuff, MAXPACKET, 0); /* TODO: check whether packet comes from the right host */
			if(n == -1)
			{
				perror("recv");
				close(s);
				return(0);
			}
			if(rcvbuff[0] == RESPONSE_DSDATA)
			{
				if(rcvbuff[1] == 1)
				{
					if(stamp) memcpy((unsigned char *) stamp, rcvbuff + 2, sizeof(time_t));
					memcpy((unsigned char *) dsd, rcvbuff + 2 + sizeof(time_t), sizeof(struct dsdata));
					close(s);
					return(1);
				}
			}
			else
				printf("Invalid reply.\n");
		}
		printf("Timeout.\n");
	}

	close(s);
	return(0);
}
