
#define PROTOCOL_VERSION 1

#define REQUEST_DSCOUNT 1
#define REQUEST_DSINFO 2
#define REQUEST_DSDATA 3

#define RESPONSE_DSCOUNT 1
#define RESPONSE_DSINFO 2
#define RESPONSE_DSDATA 3

/*

REQUEST

   +----+
   | 1  |
   +----+
   +----+----+
   | 2  | n  | (n - data source number)
   +----+----+
   +----+----+----+----+....+----+
   | 3  | n  | a1 | a2 |    | an | (n - number of items, an - number of nth data source)
   +----+----+----+----+....+----+
   
RESPONSE

   +----+----+----+
   | 1  | p  | n  | (n - data source count, p - proto version)
   +----+----+----+
   +----+----+----+------+-------------+
   | 2  | n  | di | name | description | (n - data source number, name, description - nul-terminated, di - struct dsinfo)
   +----+----+----+------+-------------+
   +----+----+-----------+----+----+....+----+
   | 3  | n  | time      | x0 | x1 |    | xn |  (n - data source count, xn - struct dsdata, time - seconds from start)
   +----+----+-----------+----+----+....+----+


*/

struct dsinfo
{
	float redBytes, redPackets;
	float yellowBytes, yellowPackets;
	float maxBytes, maxPackets;
};

struct dsdata
{
	unsigned char id, pad0, pad1, pad2; // 4
	unsigned long long overallBytes; // 8
	unsigned long overallPackets;    // 4
	float currentBytes, currentPackets; // 8
};

struct dseinfo
{
	struct dsinfo dsi;
	unsigned char * shortname;
	unsigned char * description;	
};

int rtn_count(u_int32_t ip, int port, int verbose);
int rtn_dsinfo(u_int32_t ip, int port, int verbose, int dsn, struct dseinfo * dse);
int rtn_poll(u_int32_t ip, int port, int verbose, int dsn, struct dsdata * dsd, time_t * stamp);
