
#include <pthread.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "rated.h"
#include "ratednet.h"
#include "lib.h"

#define MAXPACKET 1200

#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#endif

void * ratedServerThread(struct rated * r)
{
	int s;
	struct sockaddr_in sin;

	socklen_t fromlen;
	struct sockaddr_in fromsin;
	int n;
	
	unsigned char pktbuff[MAXPACKET];
	
	s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if(s == -1)
	{
		perror("socket");
		exit(1);
	}
	
	sin.sin_family = PF_INET;
	sin.sin_port = htons(r->udpport);
	sin.sin_addr.s_addr = INADDR_ANY;

	if(bind(s, &sin, sizeof(struct sockaddr_in)) == -1)
	{
		perror("bind");
		exit(1);
	}
	
	fromlen = sizeof(struct sockaddr_in);

	while((n = recvfrom(s, pktbuff, MAXPACKET, 0, &fromsin, &fromlen)) >= 0)
	{
		if(fromlen == sizeof(struct sockaddr_in))
		{
			unsigned char msgtype = pktbuff[0];
			struct grant * g = r->acl;
			int permission = 0;
			unsigned char response[MAXPACKET];
			int resplen = 0;
			
			for(; g; g = g->next)
				if((fromsin.sin_addr.s_addr & g->mask) == g->ip) permission = 1;
	
//			printf("Permission: %d (%s)\n", permission, inet_ntoa(sin.sin_addr));	
			if(!permission) continue;
			
			switch(msgtype)
			{
				case REQUEST_DSCOUNT:
					response[0] = RESPONSE_DSCOUNT;
					response[1] = PROTOCOL_VERSION;
					response[2] = r->sp_count;
					resplen = 3;
					break;
				case REQUEST_DSINFO:
				{
					struct dsinfo di;
					int dn = pktbuff[1];
					
					if(dn >= 0 && dn < r->sp_count)
					{
						int b;
						
						di.redBytes = r->entries[dn]->redBytes;
						di.redPackets = r->entries[dn]->redPackets;
						di.yellowBytes = r->entries[dn]->yellowBytes;
						di.yellowPackets = r->entries[dn]->yellowPackets;
						di.maxBytes = r->entries[dn]->maxBytes;
						di.maxPackets = r->entries[dn]->maxPackets;
	
						response[0] = RESPONSE_DSINFO;
						response[1] = dn;
						resplen = 2;
						memcpy(response + resplen, (char*) &di, sizeof(struct dsinfo));
						resplen += sizeof(struct dsinfo);
						
						memcpy(response + resplen, r->entries[dn]->shortname, b = min(50, strlen(r->entries[dn]->shortname)));
						resplen += b;
						
						response[resplen++] = 0;

						memcpy(response + resplen, r->entries[dn]->description, b = min((MAXPACKET - resplen) - 2, strlen(r->entries[dn]->description)));
						resplen += b;
						
						response[resplen++] = 0;
					}
					break;
				}
				case REQUEST_DSDATA:
				{
					int dc;
					dc = pktbuff[1];
					if(dc > 0 && dc < 40)
					{
						time_t t;
						struct dsdata dd;
						int i, n;
						
						response[0] = RESPONSE_DSDATA;
						resplen = 2;
						t = time(NULL) - r->startTime.time;
						memcpy(response + resplen, &t, sizeof(time_t));
						resplen += sizeof(time_t);
						
						for(i = 0, n = 0; i < dc; i++)
						{
							int dn = pktbuff[i + 2];
							
//							printf("out: %d\n", dn);
							
							if(dn >= 0 && dn < r->sp_count)
							{
								dd.id = dn;
								dd.overallBytes = r->entries[dn]->overallBytes;
								dd.overallPackets = r->entries[dn]->overallPackets;
								dd.currentBytes = r->entries[dn]->currentBytes;
								dd.currentPackets = r->entries[dn]->currentPackets;
								memcpy(response + resplen, (char*) &dd, sizeof(struct dsdata));
								resplen += sizeof(struct dsdata);
								n++;
							}
						}
						response[1] = n;
					}
					break;
				}
			}
			
			if(resplen)
				sendto(s, response, resplen, 0, &fromsin, sizeof(struct sockaddr_in));
		}
		fromlen = sizeof(struct sockaddr_in);

	}
	perror("recvfrom");
	exit(1);
}


int ratedServer(struct rated * r)
{
	pthread_t t;
	return(pthread_create(&t, NULL, (void* (*)(void*)) ratedServerThread, (void*) r));
}
